@@ -16,12 +16,15 @@
 # specific language governing permissions and limitations
 # under the License.
 #
+import json
 import logging
 import os
 import pexpect
 import re
+import stat
 import tempfile
 
+from base64 import b64encode
 from uuid import uuid4
 try:
     from collections.abc import Mapping
@@ -33,12 +36,14 @@ from six import iteritems, string_types
 from ansible_runner import defaults
 from ansible_runner.output import debug
 from ansible_runner.exceptions import ConfigurationError
+from ansible_runner.defaults import registry_auth_prefix
 from ansible_runner.loader import ArtifactLoader
 from ansible_runner.utils import (
     open_fifo_write,
     args2cmdline,
     sanitize_container_name,
-    cli_mounts
+    cli_mounts,
+    register_for_cleanup,
 )
 
 logger = logging.getLogger('ansible-runner')
@@ -58,7 +63,7 @@ class BaseConfig(object):
                  private_data_dir=None, host_cwd=None, envvars=None, passwords=None, settings=None,
                  project_dir=None, artifact_dir=None, fact_cache_type='jsonfile', fact_cache=None,
                  process_isolation=False, process_isolation_executable=None,
-                 container_image=None, container_volume_mounts=None, container_options=None, container_workdir=None,
+                 container_image=None, container_volume_mounts=None, container_options=None, container_workdir=None, container_auth_data=None,
                  ident=None, rotate_artifacts=0, timeout=None, ssh_key=None, quiet=False, json_mode=False, check_job_event_data=False):
         # common params
         self.host_cwd = host_cwd
@@ -71,6 +76,8 @@ class BaseConfig(object):
         self.container_image = container_image or defaults.default_container_image
         self.container_volume_mounts = container_volume_mounts
         self.container_workdir = container_workdir
+        self.container_auth_data = container_auth_data
+        self.registry_auth_path = None
         self.container_name = None  # like other properties, not accurate until prepare is called
         self.container_options = container_options
         self._volume_mount_paths = []
@@ -140,7 +147,7 @@ class BaseConfig(object):
         self.runner_mode = runner_mode
         try:
             if self.settings and isinstance(self.settings, dict):
-                self.settings = self.settings.update(self.loader.load_file('env/settings', Mapping))
+                self.settings.update(self.loader.load_file('env/settings', Mapping))
             else:
                 self.settings = self.loader.load_file('env/settings', Mapping)
         except ConfigurationError:
@@ -150,7 +157,7 @@ class BaseConfig(object):
         if self.runner_mode == 'pexpect':
             try:
                 if self.passwords and isinstance(self.passwords, dict):
-                    self.passwords = self.passwords.update(self.loader.load_file('env/passwords', Mapping))
+                    self.passwords.update(self.loader.load_file('env/passwords', Mapping))
                 else:
                     self.passwords = self.passwords or self.loader.load_file('env/passwords', Mapping)
                 self.expect_passwords = {
@@ -187,6 +194,7 @@ class BaseConfig(object):
         self.container_image = self.settings.get('container_image', self.container_image)
         self.container_volume_mounts = self.settings.get('container_volume_mounts', self.container_volume_mounts)
         self.container_options = self.settings.get('container_options', self.container_options)
+        self.container_auth_data = self.settings.get('container_auth_data', self.container_auth_data)
 
         if self.containerized:
             self.container_name = "ansible_runner_{}".format(sanitize_container_name(self.ident))
@@ -443,8 +451,8 @@ class BaseConfig(object):
         new_args.extend(["--workdir", workdir])
 
         # For run() and run_async() API value of base execution_mode is 'BaseExecutionMode.NONE'
-        # and the container volume mounts are handled seperately using 'container_volume_mounts'
-        #  hence ignore additonal mount here
+        # and the container volume mounts are handled separately using 'container_volume_mounts'
+        # hence ignore additional mount here
         if execution_mode != BaseExecutionMode.NONE:
             if execution_mode == BaseExecutionMode.ANSIBLE_COMMANDS:
                 self._handle_ansible_cmd_options_bind_mounts(new_args, cmdline_args)
@@ -485,6 +493,20 @@ class BaseConfig(object):
             # custom show paths inside private_data_dir do not make sense
             self._update_volume_mount_paths(new_args, "{}".format(self.private_data_dir), dst_mount_path="/runner", labels=":Z")
 
+        if self.container_auth_data:
+            # Pull in the necessary registry auth info, if there is a container cred
+            self.registry_auth_path, registry_auth_conf_file = self._generate_container_auth_dir(self.container_auth_data)
+            if 'podman' in self.process_isolation_executable:
+                new_args.extend(["--authfile={}".format(self.registry_auth_path)])
+            else:
+                docker_idx = new_args.index(self.process_isolation_executable)
+                new_args.insert(docker_idx + 1, "--config={}".format(self.registry_auth_path))
+            if registry_auth_conf_file is not None:
+                # Podman >= 3.1.0
+                self.env['CONTAINERS_REGISTRIES_CONF'] = registry_auth_conf_file
+                # Podman < 3.1.0
+                self.env['REGISTRIES_CONFIG_PATH'] = registry_auth_conf_file
+
         if self.container_volume_mounts:
             for mapping in self.container_volume_mounts:
                 volume_mounts = mapping.split(':', 2)
@@ -516,6 +538,43 @@ class BaseConfig(object):
         logger.debug(f"container engine invocation: {' '.join(new_args)}")
         return new_args
 
+    def _generate_container_auth_dir(self, auth_data):
+        host = auth_data.get('host')
+        token = "{}:{}".format(auth_data.get('username'), auth_data.get('password'))
+        encoded_container_auth_data = {'auths': {host: {'auth': b64encode(token.encode('UTF-8')).decode('UTF-8')}}}
+        # Create a new temp file with container auth data
+        path = tempfile.mkdtemp(prefix='%s%s_' % (registry_auth_prefix, self.ident))
+        register_for_cleanup(path)
+
+        if self.process_isolation_executable == 'docker':
+            auth_filename = 'config.json'
+        else:
+            auth_filename = 'auth.json'
+        registry_auth_path = os.path.join(path, auth_filename)
+        with open(registry_auth_path, 'w') as authfile:
+            os.chmod(authfile.name, stat.S_IRUSR | stat.S_IWUSR)
+            authfile.write(json.dumps(encoded_container_auth_data, indent=4))
+
+        registries_conf_path = None
+        if auth_data.get('verify_ssl', True) is False:
+            registries_conf_path = os.path.join(path, 'registries.conf')
+
+            with open(registries_conf_path, 'w') as registries_conf:
+                os.chmod(registries_conf.name, stat.S_IRUSR | stat.S_IWUSR)
+
+                lines = [
+                    '[[registry]]',
+                    'location = "{}"'.format(host),
+                    'insecure = true',
+                ]
+
+                registries_conf.write('\n'.join(lines))
+
+        auth_path = authfile.name
+        if self.process_isolation_executable == 'docker':
+            auth_path = path  # docker expects to be passed directory
+        return (auth_path, registries_conf_path)
+
     def wrap_args_with_ssh_agent(self, args, ssh_key_path, ssh_auth_sock=None, silence_ssh_add=False):
         """
         Given an existing command line and parameterization this will return the same command line wrapped with the
